using System;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;


namespace MDXTutorial03
{
	public class MDXTutorial03 : Form
	{
		Device device = null;
		PresentParameters presentParams = new PresentParameters();

		Mesh Object;

		public MDXTutorial03()
		{
			
			this.ClientSize = new System.Drawing.Size(640,480);
		
			this.Text = "MDXTutorial09: Indexbuffer";
			
		}

		public bool InitializeGraphics()
		{
			try
			{
				
				presentParams.Windowed=true;
				presentParams.SwapEffect = SwapEffect.Discard;

				device = new Device(0, DeviceType.Hardware, this, CreateFlags.SoftwareVertexProcessing, presentParams);

				device.DeviceReset += new System.EventHandler(this.OnResetDevice);
				this.OnCreateDevice(device, null);
				this.OnResetDevice(device, null);

				return true;
			}
			catch (DirectXException)
			{ 
				return false; 
			}
		}
		public void OnCreateDevice(object sender, EventArgs e)
		{
			Device dev = (Device)sender;

			CustomVertex.PositionOnly[] verts = { 			
										new CustomVertex.PositionOnly(-1f,  0f, -1f),
										new CustomVertex.PositionOnly( 1f,  0f, -1f),
										new CustomVertex.PositionOnly( 1f,  0f,  1f),
										new CustomVertex.PositionOnly(-1f,  0f,  1f),
			};

			short[] indices = { 0,1,2,
								  0,2,3 };

			Object = new Mesh(2,verts.Length,MeshFlags.WriteOnly,CustomVertex.PositionOnly.Format,
							  device);

			Object.VertexBuffer.SetData(verts,0,LockFlags.None);
			Object.IndexBuffer.SetData(indices,0,LockFlags.None);

		}
		public void OnResetDevice(object sender, EventArgs e)
		{
			Device dev = (Device)sender;

			dev.RenderState.Lighting = false;
			dev.RenderState.DitherEnable = true;
		}

		private void Render()
		{
			if (device == null) 
				return;

			device.Clear(ClearFlags.Target , System.Drawing.Color.Blue, 1.0f, 0);
			
			device.BeginScene();

			device.Transform.View = Matrix.LookAtLH( new Vector3( 0.0f, 3.0f,-5.0f ), new Vector3( 0.0f, 0.0f, 0.0f ), new Vector3( 0.0f, 1.0f, 0.0f ) );
			device.Transform.Projection = Matrix.PerspectiveFovLH( (float)Math.PI / 4, 1.0f, 1.0f, 100.0f );

			device.Transform.World = Matrix.Identity;
			
			device.RenderState.CullMode = Cull.Clockwise;
			Object.DrawSubset(0);

			device.EndScene();
			device.Present();
		}


		protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
		{
			this.Render(); 
		}

		protected override void OnResize(System.EventArgs e)
		{
			
		}
		protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
		{
			if(keyData == Keys.Escape)
				Application.Exit();

			return base.ProcessCmdKey( ref msg, keyData );
		}


		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		static void Main() 
		{
			using (MDXTutorial03 frm = new MDXTutorial03())
			{
				if (!frm.InitializeGraphics()) // Initialize Direct3D
				{
					MessageBox.Show("Could not initialize Direct3D.  This tutorial will exit.");
					return;
				}
				frm.Show();

				
				while(frm.Created)
				{
					frm.Render();
					Application.DoEvents();
				}
			}
		}

		private void InitializeComponent()
		{
			// 
			// MDXTutorial03
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Name = "MDXTutorial03";
		}

		
	}
}